@@ -8,13 +8,13 @@ Weather::Product - routines for parsing WMO-style weather products
 
 =head1 DESCRIPTION
 
-Weather::Product is a base module for parsing WMO-style weather products
-(as in forecasts, observations, etc.) as might come from various weather
-services.
+Weather::Product is a base module for parsing World Meterological Assication
+(WMO) standard weather products (as in forecasts, observations, etc.) as
+might come from various weather services.
 
 More "sophisiticated" parsing of U.S. National Weather Service (NWS)
-weather products (by AWIPS IDs or zones) is done in the
-Weather::Product::NWS module (which inherits methods from this module).
+weather products (by AWIPS Product Identifiers or zones) is done in the
+F<Weather::Product::NWS> module (which inherits methods from this module).
 
 =head1 EXAMPLE
 
@@ -28,8 +28,8 @@ Weather::Product::NWS module (which inherits methods from this module).
 
 =cut
 
-use vars qw($VERSION);
-$VERSION = "1.2.0";
+use vars qw($VERSION $AUTOLOAD);
+$VERSION = "1.2.1";
 
 @ISA = qw(Exporter);
 @EXPORT = qw();
@@ -54,7 +54,7 @@ or,
 
     $obj = new Weather::Product LIST
 
-where LIST contains file names or URLs of weather products to I<import>.
+where C<LIST> contains file names or URLs of weather products to I<L<"import">>.
 
 =cut
 
@@ -68,6 +68,14 @@ sub new {
     return $self;
 }
 
+=pod
+
+=head2 initialize
+
+An internal method called by I<L<"new">>.
+
+=cut
+
 sub initialize {
     my $self = shift;
 
@@ -75,6 +83,7 @@ sub initialize {
     $self->{products} = ();		# products parsed
     $self->{text} = ();			# text
     $self->{WMO} = undef;
+    $self->{max_age} = 0;		# maxmimum age (in hours)
 }
 
 =pod
@@ -83,16 +92,19 @@ sub initialize {
 
     $obj->import LIST
 
-"Imports"  and parses files or URLs containing weather products. This is used
-by the I<new> method if files or URLs are specified.
+Imports  and parses files or URLs containing weather products. This is used
+by the I<L<"new">> method if files or URLs are specified.
 
 I<import> can be used to update the object with a newer weather product or
 to merge additional parts, addendums, etc. (not implemented in this version).
 
-Be aware that the I<parse> method called by I<import> will not allow incompatible
-(read: "different") weather products to be imported.  You should create a new
+Be aware that the I<L<"parse">> method called by I<import> will not allow incompatible
+(read: "different") weather products to be imported. You should create a new
 object for each product.
 
+In other words, "FPUS51" and "FPUS61" are different products, requiring different
+instances of F<Weather::Product>.
+
 =cut
 
 sub import {
@@ -138,9 +150,14 @@ sub import {
 
     $obj->parse STRING
 
-"Parses" the weather product (contained in I<STRING>). Assumes I<STRING> is
-a valid weather product (containing a valid WMO header line). This is used
-by the I<import> method (and indirectly by I<new>).
+Parses the weather product (contained in C<STRING>) into a WMO header
+(see F<Weather::WMO>) and associated text. Product names derived from
+the WMO header are added to the I<L<"products">> list which link to
+the associated text.
+
+Assumes C<STRING> is a valid weather product (containing a valid WMO header
+line). This is used by the I<L<"import">> method (and indirectly by
+I<L<"new">>).
 
 =cut
 
@@ -182,6 +199,17 @@ sub parse {
     $self->purge();
 }
 
+=pod
+
+=head2 add
+
+    $obj->add PRODUCT, LINK
+
+An internal method for adding pointers to products, used by I<L<"parse">>.
+For more information on the gory details, see I<L<"pointer">>.
+
+=cut
+
 sub add {
     my $self = shift;
     my ($name, $where) = @_;
@@ -190,19 +218,40 @@ sub add {
 
 =pod
 
+=head2 age
+
+    $obj->age(PRODUCT)
+
+Returns the age (in hours) of the specified C<PRODUCT>.
+
+See I<L<"max_age">> to set a maximum allowable age.
+
+=cut
+
+sub age {
+    my $self = shift;
+    my $id = shift;
+    return (time-$self->time($id)) / 3600;
+}
+
+=pod
+
 =head2 purge
 
     $obj->purge()
 
 Purges "orphaned" weather products. In other words, garbage collection.
+(See I<L<"pointer">> for information how data is stored.)
+
+Weather products older than I<L<"max_age">> are also removed.
 
 This will only occur when another weather product (presumably an updated one)
-is imported into the object (using I<import> and I<parse>).
+is imported into the object (using I<L<"import">> and I<L<"parse">>).
 
 
     $obj->purge LIST
 
-Purges the weather products specified in LIST, if they exist. Orphaned
+Purges the weather products specified in C<LIST>, if they exist. Orphaned
 products will also be purged.
 
 =cut
@@ -210,7 +259,23 @@ products will also be purged.
 sub purge {
     my $self = shift;
 
-    foreach (@_)
+    # get user-specified list of products to purge
+    my @products_purge = @_;
+
+    # if max_age != 0, add prodcuts that are too old to this list
+    if ($self->{max_age})
+    {
+        my @products_all = $self->products();
+        foreach (@products_all)
+        {
+            if ($self->age($_)>$self->{max_age}) {
+                push @products_purge, $_;
+            }
+        }
+    }
+
+    # purge this list
+    foreach (@products_purge)
     {
         if (defined($self->{products}->{$_}))
         {
@@ -218,7 +283,8 @@ sub purge {
         }
     }
 
-    my @products = values %{$self->{products}},
+    # remove orphaned references
+    @products_purge = values %{$self->{products}},
        $ptr;
     foreach $ptr (keys %{$self->{data}}) {
         unless (grep /^$ptr$/, @products) {
@@ -231,18 +297,16 @@ sub purge {
 
 =head2 pointer
 
-    $obj->pointer PRODUCT, [FIELD]
+    $obj->pointer PRODUCT, FIELD
 
-This is an internal method for returning a "pointer" to a product (or to a
-specified "field"). It is not meant to be used outside of this module.
+This is an internal method for returning a "pointer" to a C<PRODUCT> (or
+to a specified C<FIELD> in that C<PRODUCT>).
 
-Ah, but here's an excuse to attempt an explanation of the gory details....
-
-Basically, all parsed products are stored in a hash called I<data>. Another
-hash called I<products> contains pointers to the appropriate entry in I<data>.
+Basically, all parsed products are stored in a hash called C<data>. Another
+hash called C<products> contains pointers to the appropriate entry in C<data>.
 
 While this structure may seem pointless (pun intended) for this module, it
-is useful for the Weather::Product::NWS module, where some of the products
+is useful for the F<Weather::Product::NWS module>, where some of the products
 (AWIPS IDs and individual zones) are linked to substrings of data.
 
 =cut
@@ -284,9 +348,9 @@ The definition of a "product" is a bit loose in this module. The WMO product
 identifier (ie, "FPUS51" or "FPCN55") is added.
 
 The reporting station (ie, "KNYC" or "CWNT") is also listed as a product
-in cases where different stations may issue products with the same
-identifier (such reports should really be handled with the
-Weather::Product::NWS module and not this one).
+in cases where different stations may issue products with the same identifier
+(such reports should really be handled with the F<Weather::Product::NWS>
+module and not this one).
 
 =cut
 
@@ -312,16 +376,17 @@ For example,
 
     $time = localtime($forecast->time('FPUS81'));
 
-Returns the timestamp of the parsed weather product.
+returns the timestamp of the parsed weather product.
 
 Now a note about these timestamps: they are converted to Perl-friendly
-time from the WMO header (using I<int_time>). WMO timestamps there are in
-the form of DDHHMM, where DD is the day of the month, and HHMM is the time,
-in UTC (we treat it as GMT here; the difference between the two is academic).
+time from the WMO header (using I<L<"int_time">>). WMO timestamps there are
+in the form of C<DDHHMM>, where C<DD> is the day of the month, and C<HHMM>
+is the time, in UTC (we treat it as GMT here; the difference between the two
+is academic).
 
 Why should you care? If for some strange reason you are parsing products
-that are a month old, you'll get inaccurate timestamps.  Weather products
-are not meant to be archived.
+from a previous month, you'll get inaccurate timestamps.  The module may
+even return a null.
 
 A future version of this module might attempt to fetch a "base time" from
 local files or URLs.
@@ -330,12 +395,19 @@ local files or URLs.
 
 sub time {
     my $self = shift;
+
     my $id = shift, $WMO;
 
     $WMO = $self->pointer($id, WMO);
 
     if (defined($WMO)) {
-        return int_time($WMO->time);
+        my $time = int_time($WMO->time);
+        if ($time>time)
+        {
+            return undef;	# this is too old
+        } else {
+            return $time;
+        }
     } else {
         return undef;
     }
@@ -348,8 +420,10 @@ sub time {
     TIME = Weather::Product::int_time WMO_TIME, BASE
 
 Attempts to convert WMO-style timestamps (found in WMO headers and UGC lines)
-to something Perl-friendly. If no BASE is specified, the current time (I<time>)
-is used.
+to something Perl-friendly. If no C<BASE> is specified, the current time
+(from the Perl function I<time>) is used.
+
+See I<L<"time">> for a note about the limitations of WMO-style timestamps.
 
 =cut
 
@@ -357,7 +431,7 @@ sub int_time {
     my ($ugc_time, $base_time) = @_;
 
     unless (defined($base_time)) {
-        $base_time = time;
+        $base_time = time();
     }
 
     my ($sec,$min,$hour,$mday,$mon,$year) = gmtime($base_time);
@@ -383,7 +457,7 @@ sub int_time {
 
     WMO = $obj->WMO(PRODUCT)
 
-Returns the WMO header of the specified product (as a Weather::WMO object).
+Returns the WMO header of the specified product (as a F<Weather::WMO> object).
 For example,
 
     $WMO = $forecast->WMO('FPUS41');
@@ -396,9 +470,26 @@ For example,
 Perhaps the most important method. This returns the actual text of the
 weather product. For example,
 
-
     print $forecast->text('FPUS41'), "\n";
 
+If I<text> returns a null value, there is no product by that name. (It is
+possible that the product has been L<purged|"purge">.)
+
+=head2 max_age
+
+When called without arguments, returns the maximum allowable age (in hours)
+for products. The default is C<0>.
+
+When called with an argument, sets the maximum allowable age. A useful setting
+might be for 30 hours.
+
+The I<L<"purge">> method will remove weather products older than
+the maximum allowable age.
+
+A potential pitfall for setting this property to too low a number is that
+products will be removed as soon as they are parsed. This may or may not be
+a good thing.
+
 =cut
 
 sub AUTOLOAD {
@@ -409,6 +500,16 @@ sub AUTOLOAD {
     my $name = $AUTOLOAD;
     $name =~ s/.*://;   # strip fully-qualified portion
 
+    if (grep(/^$name$/,	# settable (global) properties
+        qw(max_age)
+    )) {
+        if (@_) {
+            return $self->{$name} = shift;
+        } else {
+            return $self->{$name};
+        }
+    }
+
     if (grep(/^$name$/,
         qw(WMO text)
     )) {
@@ -434,12 +535,15 @@ __END__
 This version of the module does not (yet) handle addendums or multi-part
 weather products.
 
-Other issues with returning the I<time> of a product are explained above,
+Other issues with returning the I<L<"time">> of a product are explained above,
 are are a limitation of the WMO header format, not this module.
 
 =head1 SEE ALSO
 
-Weather::WMO and Weather::Product::NWS.
+F<Weather::WMO> and F<Weather::Product::NWS>.
+
+Perusing documentation on the format of WMO-style weather products online from
+the U.S. National Weather Service http://www.nws.noaa.gov may also be of help.
 
 =head1 DISCLAIMER
 
